/* keyboard.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 *
 * A part of this file comes from the gnome keyboard capplet (control-center):
 * Copyright (C) 2003 Sergey V. Oudaltsov <svu@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "main.h"
#include "utils.h"


/* Global iter to ease populate of treeview */
GtkTreeIter  iter, child;
GtkTreePath *selpath;
gboolean     found = FALSE;
gchar       *layout = NULL;
gchar       *variant = NULL;

/* from the gnome control center keyboard applet */
char *xci_desc_to_utf8 (XklConfigItem * ci)
{
	char *sd = g_strstrip (ci->description);
	return sd[0] == 0 ? g_strdup (ci->name) :
	    g_locale_to_utf8(sd, -1, NULL, NULL, NULL);
}

static void
add_variant_to_available_layouts_tree (XklConfigRegistry * config_registry,
				      XklConfigItem * config_item,
				      GtkTreeStore * treestore)
{
	char *utf_variant_name = xci_desc_to_utf8 (config_item);

	gtk_tree_store_append (treestore, &child, &iter);
	gtk_tree_store_set (treestore, &child,
			   AVAIL_LAYOUT_TREE_COL_DESCRIPTION, utf_variant_name,
			   AVAIL_LAYOUT_TREE_COL_ID, config_item->name, -1);
    if (config_item->name && variant
        && found && strcmp (variant, config_item->name) == 0) {
            selpath = gtk_tree_model_get_path (GTK_TREE_MODEL (treestore), &child);
    }

	g_free (utf_variant_name);
}

static void
add_layout_to_available_layouts_tree (XklConfigRegistry * config_registry,
				     XklConfigItem * config_item,
				     GtkTreeStore * treestore)
{
	char *utf_layout_name = xci_desc_to_utf8 (config_item);

	gtk_tree_store_append (treestore, &iter, NULL);
	gtk_tree_store_set (treestore, &iter,
			   AVAIL_LAYOUT_TREE_COL_DESCRIPTION, utf_layout_name,
			   AVAIL_LAYOUT_TREE_COL_ID, config_item->name, -1);

    if (strcmp (layout, config_item->name) == 0) {
        selpath = gtk_tree_model_get_path (GTK_TREE_MODEL (treestore), &iter);
        found = TRUE;
    } else
        found = FALSE;

	g_free(utf_layout_name);

	xkl_config_registry_foreach_layout_variant (config_registry,
						   config_item->name,
						   (ConfigItemProcessFunc)
						   add_variant_to_available_layouts_tree,
						   treestore);
}

void
layout_dialog_run ()
{
	GtkWidget *new_dialog, *scrolledw;
	GtkTreeStore *treestore;
	GtkWidget *treeview;
	GtkTreeSelection *selection;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeModel *model;
	GtkTreePath *path;
    gchar *code;
    gchar **explode;

    code = g_strdup (get_kbd_value ());
    explode = g_strsplit (code, "_", 0);
    g_free (code);
    layout = g_strdup (explode[0]);
    if (explode[1])
        variant = g_strdup (explode[1]);
    g_strfreev (explode);

	new_dialog =
	    xfce_titled_dialog_new_with_buttons (_("Linutop - Keyboard"),
						GTK_WINDOW (dialog->dialog),
						GTK_DIALOG_MODAL |
						GTK_DIALOG_DESTROY_WITH_PARENT,
						GTK_STOCK_CANCEL,
						GTK_RESPONSE_CANCEL,
						GTK_STOCK_OK, GTK_RESPONSE_OK,
						NULL);
	gtk_window_set_icon_name (GTK_WINDOW (new_dialog), "xfce4-keyboard");
	xfce_titled_dialog_set_subtitle ((XfceTitledDialog *) new_dialog,
					_("Select a keyboard layout"));

	treestore = gtk_tree_store_new (2, G_TYPE_STRING, G_TYPE_STRING);

	xkl_config_registry_foreach_layout (registry, (ConfigItemProcessFunc)
					   add_layout_to_available_layouts_tree,
					   treestore);

	renderer = gtk_cell_renderer_text_new ();

	column = gtk_tree_view_column_new_with_attributes (NULL,
							  renderer,
							  "text",
							  AVAIL_LAYOUT_TREE_COL_DESCRIPTION,
							  NULL);

    treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (treestore));
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (treeview), selpath);
    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (treeview), selpath,
                                  NULL, TRUE, 0.5, 0.5);
	gtk_tree_selection_select_path (selection, selpath);
    gtk_tree_path_free (selpath);

	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (treestore),
					     0, GTK_SORT_ASCENDING);

	scrolledw = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (new_dialog)->vbox),
			  scrolledw);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledw),
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
	gtk_widget_show (scrolledw);

	gtk_container_add (GTK_CONTAINER (scrolledw), treeview);

	g_signal_connect_swapped (G_OBJECT (treeview), "row-activated",
			 G_CALLBACK (cb_row_activated), new_dialog);

	gtk_widget_show (GTK_WIDGET (treeview));

	gtk_window_set_default_size (GTK_WINDOW (new_dialog), 400, 400);
	gtk_widget_show (new_dialog);

	int response = gtk_dialog_run (GTK_DIALOG (new_dialog));

    g_free (layout);
    layout = NULL;
    g_free (variant);
    variant = NULL;

	if (response == GTK_RESPONSE_OK) {
		selection =
		    gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
		gchar *id;
		gchar *ret = NULL;
        gchar *layout = NULL, *variant = NULL;

		gtk_tree_selection_get_selected (selection, &model, &iter);
		gtk_tree_model_get (model, &iter, AVAIL_LAYOUT_TREE_COL_ID, &id,
				   -1);

		path = gtk_tree_model_get_path (model, &iter);

		if (gtk_tree_path_get_depth (path) == 1) {
			layout = g_strdup (id);
			variant = g_strdup ("");
		} else {
			variant = g_strdup (id);
			gtk_tree_path_up (path);
			gtk_tree_model_get_iter (model, &iter, path);
			gtk_tree_model_get (model, &iter,
					   AVAIL_LAYOUT_TREE_COL_ID, &id, -1);
			layout = g_strdup (id);
		}
        gtk_tree_path_free (path);
		g_free (id);

		if (strcmp (variant, "") != 0)
			ret = g_strconcat (layout, "_", variant, NULL);
		else
			ret = g_strdup (layout);

        g_free (layout);
        g_free (variant);

		apply_kbd_config ((const gchar *)ret);
        set_kbd (ret);
        set_kbd_user_defined (TRUE);
        g_free (ret);
	}
	gtk_widget_destroy (new_dialog);
}
