/* lock.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <libxfce4util/libxfce4util.h>

#define LOCKCONFFILE "/cdrom/lock.cfg"

struct Config {
	gchar *autolog;
	gchar *xfcekiosk;
};
struct Config *config;

enum {
	ID_AUTOLOG = 0,
	ID_XFCEKIOSK,
};

static void
read_config ()
{
	XfceRc *rc;

	rc = xfce_rc_simple_open (LOCKCONFFILE, TRUE);
	if (rc) {
		config->autolog =
		    g_strdup (xfce_rc_read_entry (rc, "autolog", "true"));
		config->xfcekiosk =
		    g_strdup (xfce_rc_read_entry (rc, "xfcekiosk", "true"));
		xfce_rc_close (rc);
		return;
	}

	/* default config */
	config->autolog = g_strdup ("true");
	config->xfcekiosk = g_strdup ("true");
}

void
write_lock_config ()
{
	XfceRc *rc;

	rc = xfce_rc_simple_open (LOCKCONFFILE, FALSE);
	if (!rc)
		return;
	xfce_rc_write_entry (rc, "autolog", config->autolog);
	xfce_rc_write_entry (rc, "xfcekiosk", config->xfcekiosk);
	xfce_rc_close (rc);
}

static void
chk_toggled (GtkWidget *chk, gpointer data)
{
	int id;
	gboolean active;

	id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (chk), "id"));
	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (chk));

	switch (id) {
	case ID_AUTOLOG:
        g_free (config->autolog);
        config->autolog = g_strdup (active ? "true" : "false");
		break;
	case ID_XFCEKIOSK:
        g_free (config->xfcekiosk);
        config->xfcekiosk = g_strdup (active ? "true": "false");
		break;
	}
}

static gboolean
vbox_vis (GtkWidget *w, GtkStateType state, gpointer data)
{
	return FALSE;
}

static void
free_config (GtkWidget *w, gpointer data)
{
    struct Config *config = (struct Config *) data;

	g_free (config->autolog);
    g_free (config->xfcekiosk);
    g_free (config);
}

GtkWidget *
lock_widget ()
{
	GtkWidget *chk, *vbox;
	GtkWidget *hbox, *label;

	config = g_new0 (struct Config, 1);
	read_config ();
	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);

	hbox = gtk_hbox_new (FALSE, 2);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
	gtk_widget_show (hbox);

	/* autolog check btn */
	chk = gtk_check_button_new ();
	g_object_set_data (G_OBJECT (chk), "id", GINT_TO_POINTER (ID_AUTOLOG));
	gtk_box_pack_start (GTK_BOX (hbox), chk, FALSE, FALSE, 2);
	if (strcmp (config->autolog, "true") == 0)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (chk), TRUE);
	g_signal_connect (G_OBJECT (chk), "toggled",
			 G_CALLBACK (chk_toggled), NULL);
	gtk_widget_show (chk);

	label = gtk_label_new ("");
	gtk_label_set_markup (GTK_LABEL (label),
			     _("Autolog a default user\n"
			       "<span size=\"small\">"
			       "This user will have limited rights and will not be "
			       "able to alter the system.</span>"));
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 2);
	gtk_widget_show (label);

	/* xfcekiosk check btn */
	hbox = gtk_hbox_new (FALSE, 2);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
	gtk_widget_show (hbox);

	chk = gtk_check_button_new ();
	g_object_set_data (G_OBJECT (chk), "id", GINT_TO_POINTER (ID_XFCEKIOSK));
	gtk_box_pack_start (GTK_BOX (hbox), chk, FALSE, FALSE, 2);
	if (strcmp (config->xfcekiosk, "true") == 0)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (chk), TRUE);
	g_signal_connect (G_OBJECT (chk), "toggled",
			 G_CALLBACK (chk_toggled), NULL);
	gtk_widget_show (chk);

	label = gtk_label_new ("");
	gtk_label_set_markup (GTK_LABEL (label),
			     _("Activate the xfce kiosk mode\n"
			       "<span size=\"small\">"
			       "The unprivileged user will not be able to perform some "
			       "actions (modify\nthe panel, shutdown the system, edit "
			       "the menu...)</span>"));
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 2);
	gtk_widget_show (label);

	g_signal_connect (G_OBJECT (vbox), "state-changed",
			 G_CALLBACK (vbox_vis), NULL);
	g_signal_connect (G_OBJECT (vbox), "destroy",
			 G_CALLBACK (free_config), config);

	return vbox;
}
