/* main.c
 * Copyright (C) 2007 Gauvain Pocentek <gpocentek@linutop.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <pwd.h>
#include <unistd.h>

#include <glib/gprintf.h>
#include <glib/gstdio.h>

#include <libxfcegui4/libxfcegui4.h>

void
version (void)
{
    g_printf ("lstart - Version %s\n", VERSION);
}

void
usage (void)
{
    version ();
    printf ("\n"
           "Usage:\n"
           "lstart [id] [options]\n"
           "   id: a group found in /etc/xdg/lstart.conf\n"
           "       or $HOME/.config/lstart.conf\n"
           "options can be:\n"
           "   -v:     show version\n"
           "   -h:     show this message\n"
           "   -n:     no startup notification\n"
           "   -u uid: run the application with given uid\n"
           "   -c cmd: run cmd\n");
}

const gchar *
get_conffile (void)
{
    gchar *tmp =
        g_strconcat (getenv ("HOME"), "/.config/lstart.conf", NULL);

    const gchar *file = (g_access (tmp, R_OK) == 0) ?
        tmp : "/etc/xdg/lstart.conf";

    g_free (tmp);
    return file;
}

int
getint (char *text)
{
    int i = 0;
    int ret;
    ret = sscanf (text, "%d", &i);
    if (ret > 0)
        return i;
    return 0;
}

int
main (int argc, char **argv)
{
    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
    gboolean ret = 1;
    if (argc < 2)
        return ret;

    const gchar *conffile = get_conffile ();

    int oc;
    int cflg = 0;
    int uid = 0;
    struct passwd *pw;
    const gchar *cmd = "";
    gboolean use_sn = TRUE;
    XfceRc *rc;

    while ( (oc = getopt (argc, argv, "nc:u:vh")) != -1)
        switch (oc) {
        case 'h':
            usage ();
            exit (0);
            break;
        case 'v':
            version ();
            exit (0);
            break;
        case 'c':
            cflg++;
            cmd = optarg;
            break;
        case 'u':
            uid = getint (optarg);
            break;
        case 'n':
            use_sn = FALSE;
            break;
        }

    gtk_init (&argc, &argv);

    if (!cflg) {
        rc = xfce_rc_simple_open (conffile, TRUE);
        if (!xfce_rc_has_group (rc, argv[argc - 1])) {
            xfce_rc_close (rc);
            return ret;
        }
        xfce_rc_set_group (rc, argv[argc - 1]);

        cmd = xfce_rc_read_entry (rc, "command", "");
        xfce_rc_close (rc);
    }

    if (strcmp (cmd, "") != 0) {
        if ( (getuid () == 0) && (uid > 0)) {
            pw = getpwuid (uid);
            setenv ("HOME", pw->pw_dir, 1);
            setuid (uid);
            seteuid (uid);
        }
        ret = xfce_exec (cmd, FALSE, use_sn, NULL);
    }

    exit (ret);
}
